import os
import numpy as np
import csv

"""
This module creates a .txt-file with name defined via variable "fileName".

The file contains points sampled on a sphere. The portion of the sphere is variable via the linspace arguments
in the functions below. 

Via the variable "uniformSampling" the user may choose whether the points shall be created in regular shape
or uniformly sampled. The difference is that in the regular shape, points close to the poles are more dense, because
cosine and sine have more values close to 0 than at -1 or 1.

Size of the points is (N,3)

    Example:    For regular sampling: choose phi=[0...Pi], theta=[0...Pi/2] to get a quarter sphere in the upper half.
                For uniform sampling: choose phi=[0...Pi], cos_theta=[0...1] to get a quarter sphere in the upper half.

"""

fileName = 'sphere.txt'
uniformSampling = True
if os.path.isfile(fileName) == False:

    'save a portion of a sphere to file'
    pointCloud = np.array([])
    nr_samples = 2500
    radius = 5

    if uniformSampling == False:
        'square root taken in order to have the user-defined number of samples given above.'
        nr_samples = int(np.sqrt(nr_samples))
        phi = np.linspace(0, 2*np.pi, nr_samples)
        theta = np.linspace(0, np.pi, nr_samples)

        for i in range(nr_samples):
            for j in range(nr_samples):
                x = radius * np.cos(phi[i]) * np.sin(theta[j])
                y = radius * np.sin(phi[i]) * np.sin(theta[j])
                z = radius * np.cos(theta[j])

                if i == 0 and j == 0:
                    pointCloud = [[x, y, z]]
                else:
                    pointCloud = np.append(pointCloud, [[x, y, z]], axis=0)
    else:
        'sample points uniformly on sphere. Avoids higher sample density at poles.'
        for i in range(nr_samples):
            phi = np.random.uniform(0.0, np.pi/8)
            cos_theta = np.random.uniform(0.5, 1.0)
            theta = np.arccos(cos_theta)

            x = radius * np.cos(phi) * np.sin(theta)
            y = radius * np.sin(phi) * np.sin(theta)
            z = radius * np.cos(theta)

            if i == 0:
                pointCloud = [[x, y, z]]
            else:
                pointCloud = np.append(pointCloud, [[x, y, z]], axis=0)

    'save as .txt file'
    with open(fileName, mode='w') as f:
        writer = csv.writer(f)
        writer.writerows(pointCloud)
    f.close()

else:
    'File already exists'
    print('Attention: File called "'+fileName+'" already exists. Did not create a file.')